package com.persagy.dptool;

import com.persagy.dptool.dto.PhysicalObject;
import com.persagy.dptool.dto.physical.Objects;
import com.persagy.dptool.dto.physical.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.*;

public class PhysicalCheckUtil {
    private static final Logger log = LoggerFactory.getLogger(PhysicalCheckUtil.class);
    public static final String[] tipsArray = {"无异常。", "无指定项目数据。", "无数据。"};
    public static final String checkPrefix = "check_";

    public String check_projects(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<Project> projects = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    projects.add(CommonUtil.jsonStrToObj(jsonStr, Project.class));
                }
            }
        }catch (Exception e){
            return "数据格式错误！无法转为Json对象。";
        }

        if(projects.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(projects, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_objects(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<Objects> objects = new ArrayList<>();
        Set<String> objIds = new HashSet<>();

        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    Objects item = CommonUtil.jsonStrToObj(jsonStr, Objects.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        objects.add(item);

                        if(item.getObj_id() != null) {
                            objIds.add(item.getObj_id());
                        }
                    }
                }
            }
        }catch (Exception e){
            return "数据格式错误！无法转为Json对象。";
        }


        if(objects.isEmpty()) {
            return tipsArray[1];
        }

        if(!objIds.isEmpty()) {
            physicalObject.objIdSet = new HashSet<>(objIds);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(objects, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_obj_infos(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<ObjectInfo> objectInfos = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    ObjectInfo item = CommonUtil.jsonStrToObj(jsonStr, ObjectInfo.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        objectInfos.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(objectInfos.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(objectInfos, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_graph_instance(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<GraphInstance> graphInstances = new ArrayList<>();
        Set<String> graphIdSet = new HashSet<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    GraphInstance item = CommonUtil.jsonStrToObj(jsonStr, GraphInstance.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        graphInstances.add(item);

                        if(item.getGraph_type() != null && item.getSequence_id() != null) {
                            graphIdSet.add(item.getGraph_type()+item.getSequence_id());
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(graphInstances.isEmpty()) {
            return tipsArray[1];
        }

        if(!graphIdSet.isEmpty()) {
            physicalObject.graphIdSet = new HashSet<>(graphIdSet);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(graphInstances, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_rel_btw_graph_period(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        StringBuilder noObjErr = new StringBuilder();
        Set<String> graphIdSet = new HashSet<>();
        List<RelBtwGraphPeriod> relBtwGraphPeriods = new ArrayList<>();
        try {
            String graphType = null, sequenceId = null;
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    RelBtwGraphPeriod item = CommonUtil.jsonStrToObj(jsonStr, RelBtwGraphPeriod.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        relBtwGraphPeriods.add(item);

                        graphType = item.getGraph_type();
                        sequenceId = item.getSequence_id();
                        if(null != graphType && sequenceId != null) {
                            graphIdSet.add(graphType+sequenceId);
                            if(physicalObject.graphIdSet != null && !physicalObject.graphIdSet.contains(graphType+sequenceId)) {
                                noObjErr.append(item.toString() + ":"+ConstantData.errorCodes[3]+", 相关表:graph_instance\n");
                            }
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(relBtwGraphPeriods.isEmpty()) {
            return tipsArray[1];
        }

        if(!graphIdSet.isEmpty()) {
            physicalObject.graphIdPeriodSet = new HashSet<>(graphIdSet);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(relBtwGraphPeriods, physicalObject);
        if(null != errorMsg) {
            noObjErr.append(errorMsg);
        }

        if(noObjErr.length() > 0) {
            return noObjErr.toString();
        }

        return tipsArray[0];
    }

    public String check_rel_btw_objs(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<RelBtwObjs> relBtwObjs = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    RelBtwObjs item = CommonUtil.jsonStrToObj(jsonStr, RelBtwObjs.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        relBtwObjs.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(relBtwObjs.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(relBtwObjs, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_infocode_scheme(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<InfocodeScheme> infocodeSchemes = new ArrayList<>();
        Set<String> infocodeSchemeIdSet = new HashSet<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    InfocodeScheme item = CommonUtil.jsonStrToObj(jsonStr, InfocodeScheme.class);
                    if(item.getScheme_id() != null) {
                        infocodeSchemeIdSet.add(item.getScheme_id());
                    }
                    infocodeSchemes.add(item);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(infocodeSchemes.isEmpty()) {
            return tipsArray[2];
        }

        if(!infocodeSchemeIdSet.isEmpty()) {
            physicalObject.infoCodeSchemeIdSet = new HashSet<>(infocodeSchemeIdSet);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(infocodeSchemes, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_infocode_scheme_follower(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<InfocodeSchemeFollower> infocodeSchemeFollowers = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    InfocodeSchemeFollower item = CommonUtil.jsonStrToObj(jsonStr, InfocodeSchemeFollower.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        infocodeSchemeFollowers.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(infocodeSchemeFollowers.isEmpty()) {
            return tipsArray[2];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(infocodeSchemeFollowers, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_infocode_scheme_detail(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<InfocodeSchemeDetail> infocodeSchemeDetails = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    InfocodeSchemeDetail item = CommonUtil.jsonStrToObj(jsonStr, InfocodeSchemeDetail.class);
                    if(physicalObject.infoCodeSchemeIdSet != null && physicalObject.infoCodeSchemeIdSet.contains(item.getScheme_id())) {
                        infocodeSchemeDetails.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(infocodeSchemeDetails.isEmpty()) {
            return tipsArray[2];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(infocodeSchemeDetails, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_ctm_dict(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<CtmDict> ctmDicts = new ArrayList<>();
        Set<String> ctmDicIdSet = new HashSet<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    CtmDict item = CommonUtil.jsonStrToObj(jsonStr, CtmDict.class);
                    if(item.getId() != null) {
                        ctmDicIdSet.add(item.getId());
                    }
                    ctmDicts.add(item);
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(ctmDicts.isEmpty()) {
            return tipsArray[2];
        }

        if(!ctmDicIdSet.isEmpty()) {
            physicalObject.ctmDicIdSet = new HashSet<>(ctmDicIdSet);
        }

        String errorMsg = CommonUtil.annotationFieldCheck(ctmDicts, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_ctm_dict_follower(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<CtmDictFollower> ctmDictFollowers = new ArrayList<>();
        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    CtmDictFollower item = CommonUtil.jsonStrToObj(jsonStr, CtmDictFollower.class);
                    if(physicalObject.pjId.equals(item.getProject_id())) {
                        ctmDictFollowers.add(item);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(ctmDictFollowers.isEmpty()) {
            return tipsArray[1];
        }

        String errorMsg = CommonUtil.annotationFieldCheck(ctmDictFollowers, physicalObject);
        if(null != errorMsg) {
            return  errorMsg;
        }

        return tipsArray[0];
    }

    public String check_ctm_node(File file, PhysicalObject physicalObject) {
        List<String> dataList = CommonUtil.readFileToList(file);
        if(dataList.isEmpty()) {
            return tipsArray[2];
        }

        List<CtmNode> ctmNodes = new ArrayList<>();
        // 字典id：该字典下的所有ctmNode
        Map<String, List<CtmNode>> dicId2Nodes = new HashMap<>();
        // 字典id：该字典下的所有专业编码
        Map<String, Set<String>> dicId2MajorCode = new HashMap<>();
        // 字典id：该字典下的所有系统编码
        Map<String, Set<String>> dicId2SystemCode = new HashMap<>();
        // 字典id：该字典下的所有设备编码
        Map<String, Set<String>> dicId2EquipCode = new HashMap<>();
        // 字典id：该字典下的所有部件编码
        Map<String, Set<String>> dicId2CompCode = new HashMap<>();

        StringBuilder eqTreeErr = new StringBuilder();

        try {
            for(String jsonStr : dataList) {
                if(jsonStr != null && jsonStr.length() > 2) {
                    CtmNode item = CommonUtil.jsonStrToObj(jsonStr, CtmNode.class);
                    ctmNodes.add(item);

                    // 对设备岗位数相关数据进行处理
                    String dictId = item.getDict_id();
                    if(dictId != null && "eq".equals(item.getType())) {
                        if(dicId2Nodes.containsKey(dictId)) {
                            dicId2Nodes.get(dictId).add(item);
                        }else {
                            dicId2Nodes.put(dictId, new ArrayList<>(Arrays.asList(item)));
                        }

                        String code = item.getCode();
                        if(code != null) {
                            Set<String> majorSet = dicId2MajorCode.get(dictId);
                            if(null == majorSet) {
                                majorSet = new HashSet<>();
                                dicId2MajorCode.put(dictId, majorSet);
                            }

                            Set<String> systemSet = dicId2SystemCode.get(dictId);
                            if(null == systemSet) {
                                systemSet = new HashSet<>();
                                dicId2SystemCode.put(dictId, systemSet);
                            }

                            Set<String> equipSet = dicId2EquipCode.get(dictId);
                            if(null == equipSet) {
                                equipSet = new HashSet<>();
                                dicId2EquipCode.put(dictId, equipSet);
                            }

                            Set<String> compSet = dicId2CompCode.get(dictId);
                            if(null == compSet) {
                                compSet = new HashSet<>();
                                dicId2CompCode.put(dictId, compSet);
                            }

                            if(code.length() == 2) {
                                if(item.getParent() == null) {
                                    // 专业码
                                    if(majorSet.contains(code)) {
                                        eqTreeErr.append(item.toString() + ConstantData.errorCodes[4] + ", code值重复。\n");
                                    }else {
                                        majorSet.add(code);
                                    }
                                }else {
                                    // 系统码
                                    if(systemSet.contains(code)) {
                                        eqTreeErr.append(item.toString() + ConstantData.errorCodes[4] + ", code值重复。\n");
                                    }else {
                                        systemSet.add(code);
                                    }
                                }
                            }else if(code.length() == 4) {
                                // 设备码
                                if(equipSet.contains(code)) {
                                    eqTreeErr.append(item.toString() + ConstantData.errorCodes[4] + ", code值重复。\n");
                                }else {
                                    equipSet.add(code);
                                }
                            }else {
                                // 其他类型
                                if(compSet.contains(code)) {
                                    eqTreeErr.append(item.toString() + ConstantData.errorCodes[4] + ", code值重复。\n");
                                }else {
                                    compSet.add(code);
                                }
                            }
                        }
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return "数据格式错误！无法转为Json对象。";
        }

        if(ctmNodes.isEmpty()) {
            return tipsArray[2];
        }

        // 进行设备编码校验，编码长度，编码去重，父级编码存在性校验
        if(!dicId2Nodes.isEmpty()) {
            String code = "";
            String parent = "";
            int codeLength = -1;
            for(String dicId : dicId2Nodes.keySet()) {
                for(CtmNode node : dicId2Nodes.get(dicId)) {
                    code = node.getCode();
                    if(null == code) {
                        continue;
                    }

                    parent = node.getParent();
                    codeLength = code.length();

                    if(codeLength == 2 ) {
                        if(parent != null && !dicId2MajorCode.get(dicId).contains(parent)) {
                            eqTreeErr.append(node.toString() + ConstantData.errorCodes[3] + ", 找不到parent值对应的记录。\n");
                        }
                    }else if(codeLength == 4) {
                        if(!dicId2SystemCode.get(dicId).contains(parent)) {
                            eqTreeErr.append(node.toString() + ConstantData.errorCodes[1] + ", parent值不能为null且对应记录必须存在。\n");
                        }
                    }else if(codeLength == 6) {
                        if(!dicId2EquipCode.get(dicId).contains(parent)) {
                            eqTreeErr.append(node.toString() + ConstantData.errorCodes[1] + ", parent值不能为null且对应记录必须存在。\n");
                        }
                    }else {
                        eqTreeErr.append(node.toString() + ConstantData.errorCodes[2] + ", 专业和系统长度为2，设备长度为4，部件长度为6。\n");
                    }
                }
            }
        }

        String errorMsg = CommonUtil.annotationFieldCheck(ctmNodes, physicalObject);
        if(null != errorMsg) {
            eqTreeErr.append(errorMsg);
        }

        if(eqTreeErr.length() > 0) {
            return eqTreeErr.toString();
        }

        return tipsArray[0];
    }

}
