package com.persagy.dptool;

import com.persagy.dptool.dialog.HelpDialog;
import com.persagy.dptool.dialog.MessageDialog;
import com.persagy.dptool.dto.PhysicalObject;
import com.persagy.dptool.dto.VenderInfo;
import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.net.URL;
import java.util.ResourceBundle;

public class PrimaryController implements Initializable {
    private static final Logger log = LoggerFactory.getLogger(PrimaryController.class);

    @FXML
    private BorderPane paneRoot;
    @FXML
    private TabPane paneTab;
    /** 配置文件目录 */
    @FXML
    public TextField txfDir;
    @FXML
    public TextField txfDirJson;
    @FXML
    public Button btnSelectDir;
    @FXML
    public TextField txfProject;
    @FXML
    public TextField txfProjectVender;
    @FXML
    public Label lblVenderTip;
    @FXML
    public Label lblVenderResult;
    @FXML
    public TextField txfObjInfosJson;
    @FXML
    public Button btnCheck;
    @FXML
    public Button btnCheckJson;
    /** 校验结果文本域 */
    @FXML
    public TextArea txaContent;
    @FXML
    public TextArea txaContentJson;
    @FXML
    public ProgressIndicator piState;
    /** 底部状态栏 */
    @FXML
    public Label lblState;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        piState.setProgress(-1);
        piState.setVisible(false);
        lblState.setText("");

        StringBuilder tableNameSBStr = new StringBuilder();
        for (int i = 0; i < ConstantData.tablesNameList.size(); i++) {
            tableNameSBStr.append("  " + (i+1) + ". " + ConstantData.tablesNameList.get(i) + "\n");
        }
        txaContentJson.setText("目前只能校验HBase数据库中，物理世界的以下数据表数据：\n" + tableNameSBStr);

    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDir(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择配置文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDir.setText(folder.getAbsolutePath());
        }
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDirJson(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择json文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDirJson.setText(folder.getAbsolutePath());
        }
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectObjInfoJsonInput(ActionEvent e) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("选择obj_infos.json文件");

        fileChooser.getExtensionFilters().addAll(
                new FileChooser.ExtensionFilter("json", "*.json"));
        File file = fileChooser.showOpenDialog(paneRoot.getScene().getWindow());

        if(null != file) {
            txfObjInfosJson.setText(file.getAbsolutePath());
        }
    }

    /**
     * 厂商库值处理
     * @param e
     */
    public void venderValueHandle(ActionEvent e) {
        lblVenderTip.setText("");
        lblVenderResult.setText("");

        String projectId = txfProjectVender.getText();

        VenderInfo venderInfo = null;
        if(projectId != null && projectId.trim().length() == 12 && projectId.startsWith("Pj")) {
            venderInfo = new VenderInfo(projectId);
        }else {
            lblVenderTip.setText("项目id不合法！");
            txfProjectVender.requestFocus();
            return;
        }

        String jsonFilePath = txfObjInfosJson.getText();
        if(jsonFilePath != null && jsonFilePath.trim().length() > 10) {
            if(!jsonFilePath.endsWith(".json")) {
                lblVenderTip.setText("文件格式无法识别！");
                txfObjInfosJson.requestFocus();
                return;
            }
            File jsonFile = new File(jsonFilePath);
            if(jsonFile.exists() && jsonFile.isFile()) {
                Task<Boolean> task = TaskFactory.venderValueProcess(this, jsonFile, venderInfo);
                new Thread(task).start();
            }else {
                lblVenderTip.setText("文件无法识别！文件不存在或已损坏");
                txfObjInfosJson.requestFocus();
                return;
            }
        }else {
            lblVenderTip.setText("文件路径不合法！");
            txfObjInfosJson.requestFocus();
            return;
        }
    }

    /**
     * 检查数据平台配置文件
     * @param e
     */
    public void checkConfig(ActionEvent e) {
        File configFile = getFile(txfDir.getText(), "config.properties");
        File propertyFile = getFile(txfDir.getText(), "property");

        if(null == configFile && propertyFile == null) {
            txaContent.setText("未找到需要校验的config.properties文件和property数据字典文件夹，请确保所选目录是否正确。");
            return;
        }

        Task<Boolean> task = TaskFactory.checkConfig(this, configFile, propertyFile);
        new Thread(task).start();
    }

    /**
     * 检查物理世界数据表json文件
     * @param e
     */
    public void checkJsonFile(ActionEvent e) {
        txaContentJson.setText("");
        String projectId = txfProject.getText();
        PhysicalObject physicalObject = null;
        if(projectId != null && projectId.trim().length() == 12 && projectId.startsWith("Pj")) {
            physicalObject = new PhysicalObject(projectId, projectId.substring(2));
        }else {
            txaContentJson.setText("项目id不合法！");
            txfProject.requestFocus();
            return;
        }

        File jsonFileDir = null;
        String filePath = txfDirJson.getText();

        if(null != filePath) {
            filePath = filePath.trim();
            jsonFileDir = new File(filePath);
            if(!jsonFileDir.exists() || jsonFileDir.isFile()) {
                txaContentJson.setText("目录地址不合法！");
                txfDirJson.requestFocus();
                return;
            }
        }else {
            txaContentJson.setText("目录地址不合法！");
        }

        log.info("【物理世界数据表json文件】----校验----开始----");
        Task<Boolean> task = TaskFactory.checkTableJsonFiles(this, physicalObject, jsonFileDir);
        new Thread(task).start();
    }

    /**
     * 根据文件路径获取文件实例
     * @param dirPath
     * @return
     */
    private File getFile(String dirPath, String fileName) {
        File file = null;

        if(dirPath != null) {
            dirPath = dirPath.trim() + File.separator + fileName;
            file = new File(dirPath);
            if(!file.exists()) {
                file = null;
            }
        }
        return file;
    }

    /**
     * 设置tab页签中的控件是否可用
     * @param disable true-可用
     * @param typeIndex 从1开始
     */
    public void setDisable(boolean disable, int typeIndex) {
        if(1 == typeIndex) {
            paneTab.setDisable(disable);
        }else if(2 == typeIndex) {
            paneTab.setDisable(disable);
        }else if(3 == typeIndex) {
            paneTab.setDisable(disable);
        }
    }

    public void showHelp(ActionEvent e) {
        new HelpDialog(paneRoot.getScene().getWindow().getX() + 20, paneRoot.getScene().getWindow().getY() + 20).showAndWait();
    }

    public void showAbout(ActionEvent e) {
        String aboutInfo = "Version：1.0\nJDK：1.8\nAuthor：cuixubin@persagy.com";
        new MessageDialog("About", aboutInfo, 300, 120, paneRoot.getScene().getWindow().getX() + 20, paneRoot.getScene().getWindow().getY() + 20).showAndWait();
    }
}
